/*
 * Copyright (c) 1999, 2000 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * frominchan.{cc,hh} -- Read packets from NodeOS InChan, and pass along.
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include "frominchan.hh"
#include "error.hh"
#include "packet.hh"
#include "confparse.hh"
#include "glue.hh"
#include "router.hh"
#include <unistd.h>

FromInChan::FromInChan()
{
	add_output();
}

FromInChan::~FromInChan()
{
}

FromInChan *
FromInChan::clone() const
{
	return new FromInChan;
}

int
FromInChan::configure(const String &, ErrorHandler *)
{
	return 0;
}

int
FromInChan::initialize(ErrorHandler *)
{
	return 0;
}

int
FromInChan::got_packet(packet_t *nodeos_packet)
{
	/*
	 * Before we can do anything (enter Click), we have to lock
	 * the router graph. Thats because Click is not rentrant, so
	 * the thread delivering this packet has to be held up until
	 * the driver thread is sleeping and there are no other packet
	 * delivery threads running in the graph.
	 */
	mutex_lock(this->router()->nodeos_mutex());
	while (this->router()->nodeos_busy())
		cond_wait(this->router()->nodeos_cond(),
			  this->router()->nodeos_mutex());
	
	this->router()->set_nodeos_busy(1);
	mutex_unlock(this->router()->nodeos_mutex());
	
	/*
	 * Make a packet out of the internal data and push it into the graph.
	 */
	Packet* p = Packet::make(nodeos_packet);
	output(0).push(p);

	/*
	 * Let someone else proceed
	 */
	mutex_lock(this->router()->nodeos_mutex());
	this->router()->set_nodeos_busy(0);
	cond_signal(this->router()->nodeos_cond());
	mutex_unlock(this->router()->nodeos_mutex());
	
	return 0;
}

/*
 * This is the upcall from the channel code. I've munged the inchan
 * to pass back the C++ object instead of the cutchan object.
 */
extern "C" void
click_FromInChan_in(an_packet_t p, void *obj)
{
	FromInChan	*inchan = (FromInChan *) obj;

	inchan->got_packet((packet_t *) p);
}

EXPORT_ELEMENT(FromInChan)
